/* NOTE: this class will _NOT_ handle resolution changes cleanly. You have been warned. :-) */

#include <cstdio>
#include <cstring>
#include "rotatescreen.h"

RotateScreen::RotateScreen()
	: fadefrom_buf(NULL), valid(false), current_screen(0), in_fade(false), fade_to_new_info(false)
{
}

RotateScreen::~RotateScreen()
{
	delete[] fadefrom_buf;
	for (unsigned i = 0; i < subscreens.size(); ++i) {
		delete[] subscreens[i].redbuf;
		delete[] subscreens[i].buf;
	}
}

bool RotateScreen::check_invalidated()
{
	if (!valid)
		return true;
	if (in_fade)
		return true;
	if (needs_update())
		return true;
	if (!can_update())
		return false;
	
	for (unsigned i = 0; i < subscreens.size(); ++i) {
		if (subscreens[i].buf == NULL || subscreens[i].screen->check_invalidated())
			return true;
	}

	return false;
}

void RotateScreen::draw(unsigned char *buf, unsigned width, unsigned height)
{
	// see line 1 for all of this
	if (fadefrom_buf == NULL) {
		fadefrom_buf = new unsigned char[width * height * 4];
	}
	for (std::vector<Subscreen>::iterator i = subscreens.begin(); i != subscreens.end(); ++i) {
		if (i->redbuf == NULL) {
			i->redbuf = new unsigned char[width * height * 4];
		}
		if (i->buf == NULL) {
			i->buf = new unsigned char[width * height * 4];
			i->screen->draw(i->buf, width, height);
		}
	}
	// end of "line 1"-code :-)
	
	bool force = false;

	if (subscreens.size() == 0) {
		valid = true;
		gettimeofday(&last_update, NULL);
		return;
	}
	
	// if we're in a fade, complete it before doing anything else
	if (in_fade) {
		struct timeval now;
		gettimeofday(&now, NULL);

		if (!fade_found_start_time) {
			fade_found_start_time = true;
			fade_started = now;
		}
		
		double elapsed_fade = double(now.tv_sec - fade_started.tv_sec) +
			double(now.tv_usec - fade_started.tv_usec) * 1.0e-6;
		
		if (elapsed_fade > 5.5 || (!fade_to_new_info && elapsed_fade > 0.5)) {
			in_fade = false;

			memcpy(buf, subscreens[current_screen].buf, width * height * 4);
		} else {
			unsigned char *sptr1, *sptr2;
			unsigned char *dptr = buf;
			int f;
			if (fade_to_new_info) {
				if (elapsed_fade < 0.5) {
					sptr1 = fadefrom_buf;
					sptr2 = subscreens[current_screen].redbuf;
					f = unsigned(elapsed_fade/0.5 * 256.0);
				} else {
					sptr1 = subscreens[current_screen].redbuf;
					sptr2 = subscreens[current_screen].buf;
					f = unsigned((elapsed_fade-0.5)/5.0 * 256.0);
				}
			} else {
				sptr1 = fadefrom_buf;
				sptr2 = subscreens[current_screen].buf;
				f = unsigned(elapsed_fade/0.5 * 256.0);
			}

			for (unsigned i = 0; i < width * height; ++i) {
				dptr[0] = sptr1[0] + (((int(sptr2[0]) - int(sptr1[0])) * f) >> 8);
				dptr[1] = sptr1[1] + (((int(sptr2[1]) - int(sptr1[1])) * f) >> 8);
				dptr[2] = sptr1[2] + (((int(sptr2[2]) - int(sptr1[2])) * f) >> 8);

				sptr1 += 4, sptr2 += 4, dptr += 4;
			}
		}
	} else {
		// determine if we want to switch screens
	
		unsigned old_current_screen = current_screen;
		int priority = -9999;  // bah :-P
		
		// push any invalidated screen first (for now)
		for (unsigned i = 0; i < subscreens.size(); ++i) {
			if (subscreens[i].screen->check_invalidated() && subscreens[i].screen->get_priority() > priority) {
				current_screen = i;
				force = true;
				priority = subscreens[i].screen->get_priority();
			}
		}

		// check if we want to go to the next screen
		if (valid && !force && needs_update()) {
			current_screen = (current_screen + 1) % subscreens.size();
			gettimeofday(&last_update, NULL);
		}

		if (!valid || current_screen != old_current_screen || subscreens[current_screen].screen->check_invalidated()) {
			// initialize a fade
			in_fade = true;
			fade_found_start_time = false;
			fade_to_new_info = force;
			
			memcpy(fadefrom_buf, subscreens[old_current_screen].buf, width * height * 4);

			if (subscreens[current_screen].screen->check_invalidated()) {
				if (fade_to_new_info) {
					subscreens[current_screen].screen->draw(subscreens[current_screen].redbuf, width, height);
				}
				subscreens[current_screen].screen->draw(subscreens[current_screen].buf, width, height);
			}
		}
	}

	if (!valid) {
		valid = true;
		gettimeofday(&last_update, NULL);
	}
}
	
// note: makes no sense if valid=false!
bool RotateScreen::needs_update()
{
	struct timeval now;
	gettimeofday(&now, NULL);

	double since = double(now.tv_sec - last_update.tv_sec) +
		double(now.tv_usec - last_update.tv_usec) * 1.0e-6;

	return (since >= 10.0);
}

/*
 * Hold all screens for minimum three seconds, hold all screens with new info
 * for at minimum eight seconds. There's a slight hack here; a screen with new
 * info _might_ already be outdated (for instance, if we do an update to a group
 * with a mistake, then correct it quick afterwards). If it's already outdated,
 * we ignore the "eight second" rule; the results will be somewhat ugly, though,
 * but it's hard to do much better without too much special code.
 */
bool RotateScreen::can_update()
{
	struct timeval now;
	gettimeofday(&now, NULL);

	double since = double(now.tv_sec - last_update.tv_sec) +
		double(now.tv_usec - last_update.tv_usec) * 1.0e-6;

	if (since < 3.0)
		return false;
	if (fade_to_new_info && subscreens.size() > 0 && !subscreens[current_screen].screen->check_invalidated() && since < 8.0)
		return false;
	return true;
}

void RotateScreen::add_screen(GenericScreen *screen)
{
	Subscreen ss;
	ss.redbuf = NULL;
	ss.buf = NULL;
	ss.screen = screen;

	subscreens.push_back(ss);
}

