#include <cstdio>
#include <algorithm>

#include "resolution.h"
#include "top10scorescreen.h"
#include "fonts.h"
#include "theme.h"

#define RANK_X 30
#define PLAYER_X 70
#define SONG_X 370
#define SCORE_X (LOGICAL_SCREEN_WIDTH - 72)
#define SONG_MAX_WIDTH (LOGICAL_SCREEN_WIDTH - 480)

Top10ScoreScreen::Top10ScoreScreen(pqxx::connection &conn, unsigned tournament)
	: conn(conn), tournament(tournament), scores_changed(conn, "scores"), valid(false)
{
}

Top10ScoreScreen::~Top10ScoreScreen()
{
}

bool Top10ScoreScreen::check_invalidated()
{
	if (!valid)
		return true;
	if (!scores_changed.get_flag())
		return false;
	scores_changed.reset_flag();

	// check that there are indeed changes, otherwise don't bother
	std::vector<TopScore> scores;
	conn.perform(FetchTopScoresForTournament(tournament, 10, &scores));
	
	for (std::vector<TopScore>::const_iterator i = scores.begin(); i != scores.end(); ++i) {
		if (seen_topscore.count(*i) == 0) {
			valid = false;
			return true;
		}
	}

	return false;
}

void Top10ScoreScreen::draw(unsigned char *buf, unsigned width, unsigned height)
{
	scores_changed.reset_flag();
	fill_background(buf, width, height);
	set_screen_size(width, height);

	// fetch the top 10 scores
	std::vector<TopScore> scores;
	conn.perform(FetchTopScoresForTournament(tournament, 10, &scores));

	{
		unsigned width = my_draw_text("Today's top 10 scores", NULL, 40.0, "mainheading");
		my_draw_text("Today's top 10 scores", buf, 40.0, "mainheading", LOGICAL_SCREEN_WIDTH/2 - width/2, 60);
	}

	// simple headings
	my_draw_text("Player", buf, 12.0, "columnheading", PLAYER_X, 100);
	my_draw_text("Song", buf, 12.0, "columnheading", SONG_X, 100);
	width = my_draw_text("Score", NULL, 12.0, "columnheading");
	my_draw_text("Score", buf, 12.0, "columnheading", SCORE_X - width/2, 100);
	
	unsigned row = 1, y = 140;
	for (std::vector<TopScore>::const_iterator i = scores.begin(); i != scores.end(); ++i) {
		char str[16];
		std::string theme_element = "data";
		std::string heading_theme_element = "rowheading";

		// print new entries in red
		if (seen_topscore.count(*i) == 0 && seen_topscore.size() > 0) {
			theme_element = "freshdata";
			heading_theme_element = "freshrowheading";
		}

		std::sprintf(str, "%u", row++);
		unsigned width = my_draw_text(str, NULL, 24.0, heading_theme_element);
		my_draw_text(str, buf, 24.0, heading_theme_element, RANK_X - width/2, y);

		my_draw_text(i->nick, buf, 24.0, theme_element, PLAYER_X, y);

		if (my_draw_text(i->title, NULL, 24.0, theme_element) > SONG_MAX_WIDTH &&
		    !i->shorttitle.empty()) {
			my_draw_text(i->shorttitle, buf, 24.0, theme_element, SONG_X, y);
		} else {
			my_draw_text(i->title, buf, 24.0, theme_element, SONG_X, y);
		}
		
		std::sprintf(str, "%u", i->score);
		width = my_draw_text(str, NULL, 24.0, theme_element);
		my_draw_text(str, buf, 24.0, theme_element, SCORE_X - width/2, y);

		y += 40;
	}
	
	valid = true;
	
	seen_topscore.erase(seen_topscore.begin(), seen_topscore.end());
	std::copy(scores.begin(), scores.end(), std::inserter(seen_topscore, seen_topscore.end()));
}

int Top10ScoreScreen::get_priority()
{
	return 5;
}
