#include <cstdio>
#include <algorithm>

#include "resolution.h"
#include "top5chosenscreen.h"
#include "fonts.h"
#include "theme.h"

#define RANK_X 30
#define SONG_X 70
#define FREQUENCY_X (LOGICAL_SCREEN_WIDTH - 55)
#define SONG_MAX_WIDTH (LOGICAL_SCREEN_WIDTH - 190)

Top5ChosenScreen::Top5ChosenScreen(pqxx::connection &conn, unsigned tournament)
	: conn(conn), tournament(tournament), scores_changed(conn, "scores"), valid(false)
{
}

Top5ChosenScreen::~Top5ChosenScreen()
{
}

bool Top5ChosenScreen::check_invalidated()
{
	if (!valid)
		return true;
	if (!scores_changed.get_flag())
		return false;
	scores_changed.reset_flag();

	// check that there are indeed changes, otherwise don't bother
	std::vector<TopChosen> scores;
	conn.perform(FetchTopChosenSongsForTournament(tournament, 5, &scores));

	for (std::vector<TopChosen>::const_iterator i = scores.begin(); i != scores.end(); ++i) {
		if (seen_topchosen.count(*i) == 0) {
			valid = false;
			return true;
		}       
	}
	
	return false;
}

void Top5ChosenScreen::draw(unsigned char *buf, unsigned width, unsigned height)
{
	scores_changed.reset_flag();
	fill_background(buf, width, height);
	set_screen_size(width, height);

	// fetch the top 5 chosen songs
	std::vector<TopChosen> scores;
	conn.perform(FetchTopChosenSongsForTournament(tournament, 5, &scores));

	{
		unsigned width = my_draw_text("Today's top 5 chosen songs", NULL, 40.0, "mainheading");
		my_draw_text("Today's top 5 chosen songs", buf, 40.0, "mainheading", LOGICAL_SCREEN_WIDTH/2 - width/2, 60);
	}

	// simple headings
	my_draw_text("Song", buf, 12.0, "columnheading", SONG_X, 100);
	width = my_draw_text("Frequency", NULL, 12.0, "columnheading");
	my_draw_text("Frequency", buf, 12.0, "columnheading", FREQUENCY_X - width/2, 100);
	
	unsigned row = 1, y = 140;
	for (std::vector<TopChosen>::const_iterator i = scores.begin(); i != scores.end(); ++i) {
		char str[16];
		std::string theme_element = "data";
		std::string heading_theme_element = "rowheading";

		// print new entries in red
		if (seen_topchosen.count(*i) == 0 && seen_topchosen.size() > 0) {
			theme_element = "freshdata";
			heading_theme_element = "freshrowheading";
		}

		std::sprintf(str, "%u", row++);
		unsigned width = my_draw_text(str, NULL, 24.0, heading_theme_element);
		my_draw_text(str, buf, 24.0, heading_theme_element, RANK_X - width/2, y);

		if (my_draw_text(i->title, NULL, 24.0, theme_element) > SONG_MAX_WIDTH) {
			my_draw_text(i->shorttitle, buf, 24.0, theme_element, SONG_X, y);
		} else {
			my_draw_text(i->title, buf, 24.0, theme_element, SONG_X, y);
		}
		
		std::sprintf(str, "%u", i->frequency);
		width = my_draw_text(str, NULL, 24.0, theme_element);
		my_draw_text(str, buf, 24.0, theme_element, FREQUENCY_X - width/2, y);

		y += 40;
	}
	
	valid = true;
	
	seen_topchosen.erase(seen_topchosen.begin(), seen_topchosen.end());
	std::copy(scores.begin(), scores.end(), std::inserter(seen_topchosen, seen_topchosen.end()));
}

